; HISTORY
;	First version: Mario Parente
;	04 Dec 2009  FM: Use get_cat_header_data and write_cat_header.
;	17 Nov 2010  FM: -add header parameter CAT_PRODUCT_VERSION
;	18 Nov 2010  FM: -Four new CAT header parameters.


function despiker, ns, nl, nb, wl, in_filename, out_filename, $
             procedure_title, gkernelsize, gkernelsdx, gkernelsdl, $
             in_pos, in_dims, in_fid, in_bbl, threshold, spike_kernel_x, $
             spike_kernel_s, robust_trim, xstart, ystart, batch=batch

procedure_title = 'DESPIKE'


;this is primarily for testing...it switches on the ouput of a binary map which represents locations where
;spikes have been detected
spike_tracker = 0  

Fname = in_filename
ENVI_OPEN_FILE, Fname, R_FID=in_fid
; This might be the destripe output so need to get header data all over again:
stat = get_cat_header_data(in_fid, NS=ns, NL=nl, NB=nb, $
	WL=wl, WAVELENGTH_UNITS=wlunits, BBL=in_bbl, FWHM=fwhm,  $
	INTERLEAVE=interleave, OFFSET=offset, $
	DATA_IGNORE_VALUE=data_ignore_value, DATA_TYPE=data_type, $
	DIMS=in_dims, DEF_BANDS=def_bands, DESCRIP=descrip, $
	MAP_INFO=map_info, $
	XSTART=xstart, YSTART=ystart, $
	CAT_START_TIME          = cat_start_time, $
	CAT_SCLK_START          = cat_sclk_start, $
	CAT_CRISM_OBSID         = cat_crism_obsid, $
	CAT_OBS_TYPE            = cat_obs_type, $
	CAT_PRODUCT_VERSION     = cat_product_version, $
	CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id, $
	CAT_BIN_MODE            = cat_bin_mode, $
	CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
	CAT_CRISM_DETECTOR_TEMP = cat_crism_detector_temp, $
	CAT_CRISM_BENCH_TEMP    = cat_crism_bench_temp, $
	CAT_CRISM_HOUSING_TEMP  = cat_crism_housing_temp, $
	CAT_SOLAR_LONGITUDE     = cat_solar_longitude, $
	CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
	CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
	CAT_TILE_WAVE_FILE      = cat_tile_wave_file, $
	CAT_SWEETSPOT_WAVE_FILE = cat_sweetspot_wave_file, $
	CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
	CAT_IR_WAVES_REVERSED   = cat_ir_waves_reversed, $
	CAT_HISTORY             = cat_history, $
	CAT_INPUT_FILES         = cat_input_files, $
	BATCH=batch )

; Once again, override in_pos to force analysis of all bands
in_pos = long(lindgen(nb))


openw, out_lun_image, out_filename, /get_lun
if (spike_tracker eq 1) then openw, out_lun_mask, out_filename + '_mask', /get_lun

filter = cat_scan(ns, nl, nb, in_fid)
if (filter.atmo_spec[0] eq -1) then return, 1

bad = where(filter.Terrain eq 0)

;ENVI status bar initialization
status_string_array = ['Input File: ' + Fname, 'Output File: ' + out_filename]
envi_report_init, status_string_array, base = status_base, title = procedure_title + ': Removing Spikes', $
                  /interupt
envi_report_inc, status_base, nl

for j = 0, nl-1 do begin

  ;Progress Bar Update
  envi_report_stat, status_base, j, nl, cancel=cancel
  if (cancel) then begin
       print, procedure_title + ': Cancel'
       envi_report_init, base = status_base, /finish
       free_lun, out_lun_image
       if(spike_tracker) then free_lun, out_lun_mask
    return, 1
  endif

  image_in = ENVI_GET_SLICE(/bil, FID=in_fid, LINE=j, POS=in_pos, XS=0, XE=ns-1)  ;spectral by x
 
  for i=0, ns-1 do begin
    image_in[i,*] = image_in[i,*] / filter.Atmo_Spec 
    image_in[i,*] = image_in[i,*] / filter.Terrain[i,j]
  endfor

  if(spike_tracker eq 1) then begin
    data = {image:image_in,spike_mask:fltarr(ns,nb)}
  endif else begin
    data = {image:image_in}
  endelse
  
  data = spectral_despike(data, threshold, SPIKE = spike_tracker)

  for i=0, ns-1 do begin
    data.image[i,*] = data.image[i,*] * filter.Atmo_Spec 
    data.image[i,*] = data.image[i,*] * filter.Terrain[i,j]
  endfor
  if(bad[0] ne -1) then data.image[bad] = 65535
  writeu, out_lun_image, data.image
  if (spike_tracker eq 1) then writeu, out_lun_mask, data.spike_mask
endfor

free_lun, out_lun_image
if (spike_tracker eq 1) then free_lun, out_lun_mask

envi_report_init, base = status_base, /finish        

out_pos = in_pos


;============================
;    HEADER PREP
;============================

if ((n_elements(def_bands) eq 3) && (min(def_bands) gt 0)) then begin
	; For header... see note in write_cat_header prolog about 
	; decrementing def_bands
	def_bands -= 1
endif else begin
	def_bands = [0,0,0]
endelse

; Append history:
cat_history = cat_append_history(cat_history, 'CCK')

; Update input file list:
if (n_elements(cat_input_files) gt 0) then cat_input_files = strtrim(cat_input_files,2)
if (  (n_elements(cat_input_files) gt 0) && (cat_input_files[0] ne '-1') $
   && (cat_input_files[0] ne '')) then begin
	cat_input_files = [cat_input_files, file_basename(in_filename)]
endif else begin
	cat_input_files = [file_basename(in_filename)]
endelse


;============================
;    WRITE HEADER
;============================

stat = write_cat_header(FNAME=out_filename, NS=ns, NL=nl, NB=nb, $
          DATA_TYPE=4, INTERLEAVE=1, OFFSET=0, $
          BBL=in_bbl, WAVELENGTH_UNIT=wlunits, WL=wl, FWHM=fwhm, $
          DATA_IGNORE_VALUE=65535.0, DEF_BANDS=def_bands, $
          DESCRIP='CAT CRISM cirrus despiked', MAP_INFO=map_info, $
          XSTART=xstart, YSTART=ystart, $
          CAT_START_TIME          = cat_start_time, $
          CAT_SCLK_START          = cat_sclk_start, $
          CAT_CRISM_OBSID         = cat_crism_obsid, $
          CAT_OBS_TYPE            = cat_obs_type, $
          CAT_PRODUCT_VERSION     = cat_product_version, $
          CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id,  $
          CAT_BIN_MODE            = cat_bin_mode, $
          CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
          CAT_CRISM_DETECTOR_TEMP = cat_crism_detector_temp, $
          CAT_CRISM_BENCH_TEMP    = cat_crism_bench_temp, $
          CAT_CRISM_HOUSING_TEMP  = cat_crism_housing_temp, $
          CAT_SOLAR_LONGITUDE     = cat_solar_longitude, $
          CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
          CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
          CAT_TILE_WAVE_FILE      = cat_tile_wave_file, $
          CAT_SWEETSPOT_WAVE_FILE = cat_sweetspot_wave_file, $
          CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
          CAT_IR_WAVES_REVERSED   = cat_ir_waves_reversed, $
          CAT_HISTORY             = cat_history, $
          CAT_INPUT_FILES         = cat_input_files, $
          BATCH=batch )

; Report problems with header if any indicated by status returned:
if ((stat ne 0) and (~keyword_set(batch))) then begin
	if (stat eq -1) then begin
		msg = 'Problem writing header for '+file_basename(atp_setup.datafile) + string(10b) $
			      + 'Required keywords for write_cat_header undefined.'
		ok = dialog_message(msg, title='Header Error')
	endif else if (stat eq -2) then begin
		msg = 'Problem writing header for '+file_basename(atp_setup.datafile) + string(10b) $
			      + 'Header already exists.'
		ok = dialog_message(msg, title='Header Error')
	endif else begin
		msg = 'Problem writing header for '+file_basename(atp_setup.datafile) + string(10b) $
			      + 'Unspecified error.'
		ok = dialog_message(msg, title='Header Error')
	endelse
endif

if (spike_tracker eq 1) then begin
	p = strpos(out_filename,'.',/reverse_search)
	if (p gt 0) then begin
		ext = strmid(out_filename, p)
		stfnm = strmid(out_filename,0,p) + '_mask' + ext
	endif else begin
		stfnm = out_filename + '_mask'
	endelse
	envi_setup_head, fname=stfnm, ns = ns, nl = nl, nb = nb, $
       data_type = 4, interleave = 1, offset = 0, wl = wl[out_pos], $
       xstart=xstart, ystart=ystart, $
       descrip='CAT ENVI destriped pixel mask', $
       data_ignore_value=65535., wavelength_unit=wlunits, /write
	envi_open_file, stfnm
endif

END
