; HISTORY
;	First version: Mario Parente
;	04 Dec 2009  FM: -Remove in_filename, in_file_basename, and
;	                  in_file_extension args. Add input_hdr arg.
;	                  Use write_cat_header for output.
;	17 Nov 2010  FM: -add header parameter CAT_PRODUCT_VERSION
;	18 Nov 2010  FM: -Four new CAT header parameters.

function destripe, ns, nl, nb, wl, $
                   input_hdr, outfile, procedure_title, $
                   gkernelsize, gkernelsdx, gkernelsdl, $
                   pos, dims, fid, bbl, xstart, ystart


in_filename = input_hdr.in_filename

out_status_string = 'Output File: ' + outfile
openw, out_lun, outfile, /get_lun

;;;;;;;;;;;;;;;;;;;;;;;FOR TESTING PURPOSES;;;;;;;;;;;;;;;;;;;
params = 0
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;Assemble string to be displayed in ENVI status window
status_string_array = ['Input File: ' + in_filename, out_status_string]
envi_report_init, status_string_array, base = status_base, title = procedure_title + ': Creating Artifact Map', $
                  /interupt
envi_report_inc, status_base, nl

;Second, generate mxp array to stack the lines
Y_Compressed=fltarr(ns,nb)            

;surge initializations
surge_protector=fltarr(nl)


good = surge(fid, ns, nl, nb, pos, bbl)
if (good[0] eq -1) then begin
	msg = 'No good data in '+file_basename(in_filename) + string(10b) $
			      + 'Aborting Destripe.'
	ok = dialog_message(msg, title='Destripe Error')
	free_lun, out_lun
	return, 1
endif


bbl_index = where(bbl)
;;;;;;;;;;;;;;;;;;;LOOP 1;;;;;;;;;;;;;;;;;;;
;create striping filter
for j = 0, nl-1 do begin
  envi_report_stat, status_base, j, nl, cancel=cancel
  if (cancel) then begin
       print, procedure_title + ': Cancel'
       envi_report_init, base = status_base, /finish
      free_lun, out_lun
    return, 1
  endif
      
  in_data = ENVI_GET_SLICE(/bil, FID=fid, LINE=j, POS=pos, XS=0, XE=ns-1)
  
  ;A number of fancier detections were attempted, however the simple cut-off used here makes it robust to
  ;highly spiked scenarios
  if(  n_elements(where(in_data[good] gt 1)) gt nb*ns/100 ) then begin
    surge_protector[j] = 1 
  endif else begin
    Y_Compressed[*,bbl_index]= in_data[*,bbl_index] + Y_Compressed[*,bbl_index]
  endelse  
endfor
Y_Compressed = Y_Compressed/( nl-n_elements(where(surge_protector)) )
envi_report_init, base = status_base, /finish        

;;;;;;;;;;;;;;;SMOOTH FILTERING IMAGE;;;;;;;;;;;;;;;;;;;;
bad = where(Y_Compressed gt 65000)
zeros = where(Y_compressed eq 0)

if(bad[0] ne -1) then Y_Compressed[bad] = !values.F_NAN
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;************************************************
;************************************************
;check = where(surge_protector) 
;if(check[0] ne -1) then $
;  Y_Compressed[where(surge_protector),*] = !values.F_NAN
;Y_Compressed[*,bbl_index] = !values.F_NAN

resid = get_stripe_resid(Y_Compressed,[gkernelsize,gkernelsdx,gkernelsdl])
if(bad[0] ne -1) then resid[bad] = 1
if(zeros[0] ne -1) then resid[zeros] = 1  ;this avoids a divide by zero problem

;;;;;;;;;;;;;;;;;LOOP 2;;;;;;;;;;;;;;;;;;;;
envi_report_init, status_string_array, base = status_base, title = procedure_title + ': Applying Destriping Filter', $
                  /interupt
envi_report_inc, status_base, nl

;Apply corrections
for j = 0, nl-1 do begin
        envi_report_stat, status_base, j, nl, cancel=cancel
        if (cancel) then begin
          print, procedure_title + ': Cancel'
          envi_report_init, base = status_base, /finish
            free_lun, out_lun
          return, 1
        endif
        
  ;Extract 1 band layer into memory 
  im = ENVI_GET_SLICE(/bil, FID=fid, LINE=j, POS=pos, XS=0, XE=ns-1)
  
  ;SURGE corrections: 

  if (surge_protector[j] EQ 1) then begin
    if(j lt nl-1) then $
      above = ENVI_GET_SLICE(/bil, FID=fid, LINE=j+1, POS=pos, XS=0, XE=ns-1)
    if(j gt 0) then $
      below = ENVI_GET_SLICE(/bil, FID=fid, LINE=j-1, POS=pos, XS=0, XE=ns-1)
    CASE j OF
      0   : im = above
      ns-1  : im = below
      ELSE: im = (above + below)/2
    ENDCASE
  endif

  ;destriping filter applied to data cube, one band at a time   
  im=im/resid  

  writeu, out_lun, im
endfor
envi_report_init, base = status_base, /finish


;============================
;    HEADER PREP
;============================

def_bands = input_hdr.def_bands
if ((n_elements(def_bands) eq 3) && (min(def_bands) gt 0)) then begin
	; For header... see note in write_cat_header prolog about 
	; decrementing def_bands
	def_bands -= 1
endif else begin
	def_bands = [0,0,0]
endelse

; Append history:
cat_history = cat_append_history(input_hdr.cat_history, 'CCS')

; Update input file list:
cat_input_files = input_hdr.cat_input_files
if (n_elements(cat_input_files) gt 0) then cat_input_files = strtrim(cat_input_files,2)
if (  (n_elements(cat_input_files) gt 0) && (cat_input_files[0] ne '-1') $
   && (cat_input_files[0] ne '')) then begin
	cat_input_files = [cat_input_files, file_basename(in_filename)]
endif else begin
	cat_input_files = [file_basename(in_filename)]
endelse
	

;============================
;    WRITE HEADER
;============================

stat = write_cat_header(FNAME=outfile, NS=ns, NL=nl, NB=nb, $
          DATA_TYPE=4, INTERLEAVE=1, OFFSET=0, $
          BBL=input_hdr.bbl, WAVELENGTH_UNIT=wlunits, WL=wl, FWHM=input_hdr.fwhm, $
          DATA_IGNORE_VALUE=65535.0, DEF_BANDS=def_bands, $
          DESCRIP='CAT CRISM cirrus destriped', MAP_INFO=input_hdr.map_info, $
          XSTART=xstart, YSTART=ystart, $
          CAT_START_TIME          = input_hdr.cat_start_time, $
          CAT_SCLK_START          = input_hdr.cat_sclk_start, $
          CAT_CRISM_OBSID         = input_hdr.cat_crism_obsid, $
          CAT_OBS_TYPE            = input_hdr.cat_obs_type, $
          CAT_PRODUCT_VERSION     = input_hdr.cat_product_version, $
          CAT_CRISM_DETECTOR_ID   = input_hdr.cat_crism_detector_id,  $
          CAT_BIN_MODE            = input_hdr.cat_bin_mode, $
          CAT_WAVELENGTH_FILTER   = input_hdr.cat_wavelength_filter, $
          CAT_CRISM_DETECTOR_TEMP = input_hdr.cat_crism_detector_temp, $
          CAT_CRISM_BENCH_TEMP    = input_hdr.cat_crism_bench_temp, $
          CAT_CRISM_HOUSING_TEMP  = input_hdr.cat_crism_housing_temp, $
          CAT_SOLAR_LONGITUDE     = input_hdr.cat_solar_longitude, $
          CAT_PDS_LABEL_FILE      = input_hdr.cat_pds_label_file, $
          CAT_SPECTRUM_RESAMPLED  = input_hdr.cat_spectrum_resampled, $
          CAT_TILE_WAVE_FILE      = input_hdr.cat_tile_wave_file, $
          CAT_SWEETSPOT_WAVE_FILE = input_hdr.cat_sweetspot_wave_file, $
          CAT_WA_WAVE_FILE        = input_hdr.cat_wa_wave_file,  $
          CAT_IR_WAVES_REVERSED   = input_hdr.cat_ir_waves_reversed, $
          CAT_HISTORY             = cat_history, $
          CAT_INPUT_FILES         = cat_input_files, $
          BATCH=batch )

; Report problems with header if any indicated by status returned:
if ((stat ne 0) and (~keyword_set(batch))) then begin
	if (stat eq -1) then begin
		msg = 'Problem writing header for '+file_basename(atp_setup.datafile) + string(10b) $
			      + 'Required keywords for write_cat_header undefined.'
		ok = dialog_message(msg, title='Header Error')
	endif else if (stat eq -2) then begin
		msg = 'Problem writing header for '+file_basename(atp_setup.datafile) + string(10b) $
			      + 'Header already exists.'
		ok = dialog_message(msg, title='Header Error')
	endif else begin
		msg = 'Problem writing header for '+file_basename(atp_setup.datafile) + string(10b) $
			      + 'Unspecified error.'
		ok = dialog_message(msg, title='Header Error')
	endelse
endif

return, 0     
end
