;Created to support mro_crism_clean_envi.pro
;    
;    Created:  July 1st(ish)
;              Trevor Clark
;
;Input: 2-D spatial layer of an image cube, dimension of the layer
;
;Output: surge_info, a structure including a list of 'bad' row,
;        and the correspodinding count of how many pixels in that row were found to be bad 
;
;Purpose: This function detects 'bad lines' in a 2D image, and returns the detection information,
;         so that they can be fixed in mro_crism_clean_envi 
;
;Operational Outline:
;        July 01, 2008  Created
;        July 15, 2008  Added in thresholds, return value changed to include count
;        July 22, 2008  Image size problem fixed, index switched
;        Aug  11, 2008  Changed return value to {reduced_surges, bad_band_flag}
;        Dec  04, 2009  FM - Allow integer bbl input, and use logical negation (~)
;                       instead of bitwise complement (not). Return -1 if no good data.
;        Feb 07, 2013  FM - Recast ns*nb to long for spike count threshold; avoids 
;                           too-low threshold problem with int mod 32768.
;
function surge, fid, ns, nl, nb, pos, bbl

;;;;;;;;;;;;;;;;;;NEW SURGE, find the good and bad values common to all x by lambda planes;;;;;;;;;;;;;;;;    
count = 1
while(count ne 0 AND count lt nl*.9) do begin
  bad_pixel_finder = ENVI_GET_SLICE(/bil, FID=fid, LINE=nl*.1+count, POS=pos, XS=0, XE=ns-1)  ;add count back in if you reinstate the while loop

  ; FM, 12/4/09 - changing 'not' operator to ~, because 'not' causes crash if 
  ; bbl is integer type. 'not' is bitwise complement of integers, so 
  ; not 0 = -1, not 1 = -2, both of which satisy the where() test, so 
  ; everything is replaced by 65535 and the program crashes later because 
  ; nothing's good.  It worked for float bbl, because not x = 0.0 if x != 0.0, 
  ; and not x = 1.0 if x=0.0, so the test works as desired. 
  ; Logical negation operator ~bbl produces the desired result in either case.
  ; So ~bbl always works, where (not bbl) only works if bbl is float.
  ; But bbl never normally float. So Change bbl to integer type and use ~.
  ;index = where(not bbl)    ;double negative, index does in fact index the BAD bands
  index = where(~bbl)    ;double negative, index does in fact index the BAD bands
  if (index[0] ne -1) then begin
    for i=0, n_elements(index)-1 do begin
      bad_pixel_finder[*,index[i]] = 65535 
    endfor
  endif

  check = where(bad_pixel_finder lt -10)
  if (check[0] ne -1) then $
    bad_pixel_finder[where(bad_pixel_finder lt -10)] = 65535
  good = where((bad_pixel_finder LT 10), ngood)
  if (ngood eq 0) then return,-1    
  ;bad  = where(bad_pixel_finder GE 10)

  image = fltarr(ns,nb)
  spike_mask = fltarr(ns, nb)
  good_mask = {image:image, spike_mask:spike_mask}
  good_mask.image[good] = 1
  good_mask = spectral_despike(good_mask, .5, SPIKE = 1)
  if (n_elements(where(good_mask.spike_mask)) lt long(ns)*long(nb)/1000L) then begin
    count = 0
  endif else begin
    count = count + 1
  endelse
  good = where(good_mask.image eq 1)
endwhile
return, good
end
