;01/26/2009 (fps)
;   Initial development
;   	Calculate linear continuum cube from selected tie points
;   	Optional keyword median_window will assign median spectral value over specified range to tie point  
;02/04/2009 (fps)
;   Return structure with continuum, ratio, and/or parameter cube as indicated by optional keywords
;   Default is still to return only the continuum cube; keyword /full_struct returns all
;02/06/2009 (fps)
;   Remove console output for use in w/ENVI tiling (repetitive function calls)
;11/05/2010 (fps)
;   Propagate CRISM NAN or specified ignore value pixels into output cubes
;

function mro_crism_continuum_cube, in_cube, tie_indx = tie_indx, lambda_vec = lambda_vec, tie_lambda = tie_lambda, $
    	    	    	    	   tie_band1 = tie_band1, tie_band2 = tie_band2, $
    	    	    	    	   continuum_cube = continuum_cube, ratio_cube = ratio_cube, parameter_cube = parameter_cube, full_struct = full_struct, $
    	    	    	    	   median_window = median_window, $ 
    	    	    	    	   ignore_val = ignore_val, winid_start = winid_start

if (not(keyword_set(ignore_val))) then begin
    ignore_val = 65535.0
endif

if (keyword_set(full_struct)) then begin
    continuum_cube = 1
    ratio_cube = 1
    parameter_cube = 1
endif

continuum_cube = keyword_set(continuum_cube)
ratio_cube = keyword_set(ratio_cube)
parameter_cube = keyword_set(parameter_cube)

if (total([continuum_cube, ratio_cube, parameter_cube]) EQ 0) then begin
    continuum_cube = 1
endif

in_cube_size = size(in_cube, /struct)
;help, in_cube_size, /struct

cube_median = median(in_cube, dimension = 3)
cube_median_ignore_hist = histogram(cube_median, min = ignore_val, max = ignore_val, reverse_indices = ri_cube_median_ignore_hist)
cube_median_ignore_band = make_array(in_cube_size.dimensions[0], in_cube_size.dimensions[1], value = 0b)
if (cube_median_ignore_hist[0] GT 0) then begin
    cube_median_ignore_band[ri_cube_median_ignore_hist[ri_cube_median_ignore_hist[0]:ri_cube_median_ignore_hist[1]-1]] = 1b 
endif

if ((keyword_set(lambda_vec)) AND (keyword_set(tie_lambda))) then begin
    tie_indx = make_array(n_elements(tie_lambda), value = -1)
    for i = 0, n_elements(tie_indx) -1 do begin
    	tie_band_delta_min = min(abs(tie_lambda[i] - lambda_vec), tie_band_delta_mindx)
	tie_indx[i] = tie_band_delta_mindx
	tie_lambda[i] = lambda_vec[tie_band_delta_mindx]
    endfor
endif else begin
    if (not(keyword_set(tie_indx))) then begin
    	print, 'Insufficient Input...
    	return, 0
    endif
    lambda_vec = findgen(in_cube_size.dimensions[2])
    tie_lambda = lambda_vec[tie_indx]
endelse


;print, tie_indx
;print, tie_lambda

if (keyword_set(median_window)) then begin
    tie_band = make_array(in_cube_size.dimensions[0], in_cube_size.dimensions[1], 2, value = ignore_val)
    tie_band[*,*,0] = median(in_cube[*,*,(tie_indx[0] - median_window / 2):(tie_indx[0] + median_window / 2)], dimension = 3)
    tie_band[*,*,1] = median(in_cube[*,*,(tie_indx[1] - median_window / 2):(tie_indx[1] + median_window / 2)], dimension = 3)
endif else begin
    tie_band = in_cube[*,*,tie_indx]
endelse

;help, tie_band

slope_band = (tie_band[*,*,0] - tie_band[*,*,1]) / (tie_lambda[0] - tie_lambda[1])
offset_band = -1.0 * slope_band * tie_lambda[0] + tie_band[*,*,0]

param_cube = make_array(in_cube_size.dimensions[0], in_cube_size.dimensions[1], 2, value = ignore_val)
param_cube[*,*,0] = offset_band
param_cube[*,*,1] = slope_band

if (cube_median_ignore_hist[0] GT 0) then begin
    param_ignore_cube = cmreplicate(cube_median_ignore_band, 2)
    param_ignore_hist = histogram(param_ignore_cube, min = 1b, max = 1b, reverse_indices = ri_param_ignore_hist)
    param_cube[ri_param_ignore_hist[ri_param_ignore_hist[0]:ri_param_ignore_hist[1]-1]] = ignore_val
endif

;help, param_cube

if (not(keyword_set(winid_start))) then begin
    winid_start = 10
endif

;mro_crism_quick_view, tie_band, band = 0, winid = winid_start + 1
;mro_crism_quick_view, tie_band, band = 1, winid = winid_start + 2
;mro_crism_quick_view, param_cube, band = 0, winid = winid_start + 3
;mro_crism_quick_view, param_cube, band = 1, winid = winid_start + 4

if (keyword_set(continuum_cube)) then begin

    cont_cube = make_array(dimension = in_cube_size.dimensions[where(in_cube_size.dimensions NE 0)], value = ignore_val)
    ;cont_cube_alt = make_array(dimension = in_cube_size.dimensions[where(in_cube_size.dimensions NE 0)], value = ignore_val)

;    help, cont_cube

    for k = 0, in_cube_size.dimensions[2] -1 do begin
    ;    cont_cube[*,*,k] = slope_band * (lambda_vec[k] - tie_lambda[0]) + tie_band[*,*,0]
    	cont_cube[*,*,k] = slope_band * lambda_vec[k] + offset_band
    endfor

    ;mro_crism_quick_view, cont_cube, winid = winid_start + 4

    if (cube_median_ignore_hist[0] GT 0) then begin
    	cont_ignore_cube = cmreplicate(cube_median_ignore_band, in_cube_size.dimensions[2])
    	cont_ignore_hist = histogram(cont_ignore_cube, min = 1b, max = 1b, reverse_indices = ri_cont_ignore_hist)
    	cont_cube[ri_cont_ignore_hist[ri_cont_ignore_hist[0]:ri_cont_ignore_hist[1]-1]] = ignore_val
    endif

endif else begin
    cont_cube = -1
endelse


if (keyword_set(ratio_cube)) then begin
    rat_cube = in_cube / cont_cube
;    help, rat_cube

    if (cube_median_ignore_hist[0] GT 0) then begin
    	rat_ignore_cube = cmreplicate(cube_median_ignore_band, in_cube_size.dimensions[2])
    	rat_ignore_hist = histogram(rat_ignore_cube, min = 1b, max = 1b, reverse_indices = ri_rat_ignore_hist)
    	rat_cube[ri_rat_ignore_hist[ri_rat_ignore_hist[0]:ri_rat_ignore_hist[1]-1]] = ignore_val
    endif

endif else begin
    rat_cube = -1
endelse

if (not(keyword_set(parameter_cube))) then begin
    param_cube = -1
endif

if (not(keyword_set(continuum_cube))) then begin
    cont_cube = -1
endif


return, {cont_cube:cont_cube, ratio_cube:rat_cube, param_cube:param_cube}

END
