;02/06/2009 (fps)
;   Initial development - ENVI interface for mro_crism_continuum_cube()
;   	Query EVNI ABL for input cube
;   	ENVI BIL tiling
;   	Output to memory or file
;02/06/2009 (fps) - Changes since sending code to RVM @ ~4:45 ET
;   Properly handle selection of cancel button on interface widget
;   Open output file in ABL with separate call to envi_open_file, as opposed to using /open option in envi_setup_head
;   	For some strange reason the latter causes z-plot issues with the resulting cube... still have issues with in-memory cubes...
;03/10/2009 (fps)
;   Added optional 'event' argument for ENVI menu support
;12/02/2009 (mfm)
;   Use get_cat_header_data and write_cat_header. Make full header when writing to file. 
;12/04/2009 (mfm)
;	Check min(def_bands) instead of total(def_bands) 
;11/17/2010 (mfm)
;	add header parameter CAT_PRODUCT_VERSION
;11/18/2010 (mfm)
;	four more CAT header parameters
;11/05/2010 (fps)
;   Fixed specification of default bands to load and band names for parameter output cube
;12/20/2010 (mfm)
;   Resolved svn conflict with last updates (mfm vs fps) - no real problem
;11/14/2011 (fps)
;   Renamed this procedure (w/CAT support) mro_crism_continuum_cube_cat_envi
;   Former procedure name mro_crism_continuum_cube_envi will be CAT independent - for testing, feline-free zones, etc.
;10/15/2015 (mfm)
;	Check whether median filter runs off the end of the selected tie range to avoid crash.
;

pro mro_crism_continuum_cube_cat_envi, event

procedure_name = 'MRO CRISM CONTINUUM CUBE'

;Set compiler options
compile_opt strictarr

;;Simple catch to prevent crash
;catch, error
;;error = 0
;if (error NE 0) then begin
;	;Convert !error_state structure to a string array
;	help, !error_state, /struct, output = error_state_report
;	help, !error_state, /struct
;
;	error_dialog = dialog_message(error_state_report, /error, title = procedure_name +': Unexpected Error')
;
;	if (strupcase(error_dialog) EQ 'OK') then return
;endif

;Select input band from available bands list
envi_select, fid=in_fid, pos=in_pos, dims=in_dims, file_type=in_file_type, $
         /file_only, /no_spec, /no_dims, title = procedure_name + ': Select Input Cube'
if (in_fid[0] eq -1) then begin
    print, procedure_name + ': Cancel/Invalid FID'
    return
endif

;goto, skip

;Gather some useful info
stat = get_cat_header_data(in_fid, NS=ns, NL=nl, NB=nb, $
	WL=wl, WAVELENGTH_UNITS=wu, BBL=bbl, FWHM=fwhm, BNAMES=bnames, FNAME=in_filename, $
	INTERLEAVE=interleave, OFFSET=offset, $
	DATA_IGNORE_VALUE=data_ignore_value, DATA_TYPE=data_type, $
	DEF_BANDS=def_bands, DESCRIP=descrip, $
	MAP_INFO=map_info_struct, $
	CAT_START_TIME          = cat_start_time, $
	CAT_SCLK_START          = cat_sclk_start, $
	CAT_CRISM_OBSID         = cat_crism_obsid, $
	CAT_OBS_TYPE            = cat_obs_type, $
	CAT_PRODUCT_VERSION     = cat_product_version, $
	CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id, $
	CAT_BIN_MODE            = cat_bin_mode, $
	CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
	CAT_CRISM_DETECTOR_TEMP = cat_crism_detector_temp, $
	CAT_CRISM_BENCH_TEMP    = cat_crism_bench_temp, $
	CAT_CRISM_HOUSING_TEMP  = cat_crism_housing_temp, $
	CAT_SOLAR_LONGITUDE     = cat_solar_longitude, $
	CAT_PDS_LABEL_FILE      = lblfile, $
	CAT_SPECTRUM_RESAMPLED  = resamp, $
	CAT_TILE_WAVE_FILE      = cat_tile_wave_file, $
	CAT_SWEETSPOT_WAVE_FILE = cat_sweetspot_wave_file, $
	CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
	CAT_IR_WAVES_REVERSED   = cat_ir_waves_reversed, $
	CAT_HISTORY             = cat_history, $
	CAT_INPUT_FILES         = cat_input_files )

;skip:

;Overmand in_pos which reflects bbl by default...
in_pos = lindgen(nb)



;print, in_fid
;print, in_filename
;print, ns,nl,nb
;help, in_pos
;help, wl
;help, bnames
;help, bbl
;print, data_ignore_value
;help, map_info_struct, /struct


data_ignore_default = 1.0E-34
ignore_val = 65535.0

if (data_ignore_value EQ  data_ignore_default) then begin
	data_ignore_value = ignore_val
endif

continuum_info_struct = mro_crism_continuum_cube_envi_gui(in_filename, nb, wavelength_vec = wl)

if (continuum_info_struct.cancel EQ 1) then begin
	print, procedure_name + ': Cancel'
	return
endif

print, continuum_info_struct
help, continuum_info_struct, /struct

result_select = continuum_info_struct.result_select
param_indx = continuum_info_struct.param_indx
param_text = continuum_info_struct.param_text
median_select = continuum_info_struct.median_select
median_window = long(continuum_info_struct.median_text)

if ((median_select EQ 0) OR (median_window EQ 1)) then begin
	cont_median_window = 0
endif else begin
	cont_median_window = median_window
endelse

if (continuum_info_struct.ref_select EQ 0) then begin
	cont_lambda_vec = wl
	cont_tie_lambda = float(param_text)
	cont_tie_indx = 0
endif else begin
	cont_lambda_vec = 0
	cont_tie_lambda = 0
	cont_tie_indx = long(param_indx)
endelse

;Set parameters
result_dims = in_dims
result_dt = 4 ;floating point
result_nb_array = [nb,nb,2]

out_status_string_array = ['Continuum: [Inactive]', $
    	    	    	   'Ratio: [Inactive]',     $
			   'Parameter: [Inactive]']

out_ptr_array = ptrarr(3, /allocate_heap)
print, ptr_valid(out_ptr_array)

active_indx = where(result_select EQ 1, num_active_indx)
for i = 0, num_active_indx -1 do begin
	print, active_indx[i]
    
	;memory check...
	mem_check = magic_mem_check(dims=result_dims, in_memory = continuum_info_struct.outfm_result_array[active_indx[i]].in_memory, out_dt=result_dt, nb=result_nb_array[active_indx[i]], $
    	    	    	    	out_name = continuum_info_struct.outfm_result_array[active_indx[i]].name)
	if (mem_check.cancel EQ 1) then begin
		print, procedure_name + ': Cancel'
		for j = 0, n_elements(out_ptr_array) -1 do begin
			ptr_free, out_ptr_array[j]
		endfor
		return
	endif

	;Store result of memory check in info structure
	continuum_info_struct.outfm_result_array[active_indx[i]].in_memory = mem_check.in_memory
	continuum_info_struct.outfm_result_array[active_indx[i]].name = mem_check.out_name

	;Declare in-memory output array or open output file
	if (continuum_info_struct.outfm_result_array[active_indx[i]].in_memory EQ 1) then begin
		*out_ptr_array[active_indx[i]] = make_array(ns, nl, result_nb_array[active_indx[i]], value = !values.f_nan, /float)
		out_status_string_array[active_indx[i]] = (strsplit(out_status_string_array[active_indx[i]], ' ', /extract))[0] + ' ' + '[In Memory]'
	endif else begin
		openw, result_lun, continuum_info_struct.outfm_result_array[active_indx[i]].name, /get_lun
		out_status_string_array[active_indx[i]] = (strsplit(out_status_string_array[active_indx[i]], ' ', /extract))[0] + ' ' + continuum_info_struct.outfm_result_array[active_indx[i]].name
		*out_ptr_array[active_indx[i]] = result_lun
	endelse       
endfor

print, transpose(out_status_string_array)

; Check for median edge overrun:
if (n_elements(cont_tie_lambda) eq 2) then begin
	tdx = intarr(2)
	for i=0,1 do begin
		dum = min(abs(float(cont_lambda_vec)-cont_tie_lambda[i]), j)
		tdx[i] = j
	endfor
	tdx = tdx[sort(tdx)]
	if (tdx[0] - cont_median_window/2 lt 0) then begin
		ok = dialog_message('Cannot compute: Median window runs off short wavelength end.', $
		      title='Continuum Cube Fail' )
		return
	endif
	if (tdx[1] + cont_median_window/2 gt nb-1) then begin
		ok = dialog_message('Cannot compute: Median window runs off long wavelength end.', $
		      title='Continuum Cube Fail' )
		return
	endif
endif
	

;Initiate BIL tiling
in_tile_id = envi_init_tile(in_fid, in_pos, num_tiles = num_in_tiles, interleave = 1, $     	    ;BIL tiling
    	    	    	    xs = in_dims[1], xe = in_dims[2], ys = in_dims[3], ye = in_dims[4])

envi_report_init, out_status_string_array, base = status_base, title = procedure_name, /interupt

envi_report_inc, status_base, num_in_tiles

;Main processing loop - one iteration for each tile
for i = long(0), num_in_tiles -1 do begin

	envi_report_stat, status_base, i, num_in_tiles, cancel=cancel
	if (cancel) then begin
		print, procedure_name + ': Cancel'
		envi_report_init, base = status_base, /finish
		file_indx = where((continuum_info_struct.outfm_result_array.in_memory EQ 0) AND (result_select EQ 1), num_file_indx)
		if (num_file_indx GT 0) then begin
			for j = 0, num_file_indx -1 do begin
				free_lun, *out_ptr_array[file_indx[j]]
			endfor
		endif
	
		for j = 0, n_elements(out_ptr_array) -1 do begin
			ptr_free, out_ptr_array[j]
		endfor
		return
	endif

	;Grab tile data
	in_data = envi_get_tile(in_tile_id, i)

	in_data = reform(in_data, n_elements(in_data[*,0]), 1, n_elements(in_data[0,*]))

	continuum_struct = mro_crism_continuum_cube(in_data, lambda_vec = cont_lambda_vec, tie_lambda = cont_tie_lambda, tie_indx = cont_tie_indx, $
    	    	    	    	    	    	continuum_cube = result_select[0], ratio_cube = result_select[1], parameter_cube = result_select[2], $
						median_window = cont_median_window)

	for j = 0, n_elements(result_select) -1 do begin
		if (result_select[j] EQ 1) then begin
			if (n_elements(*out_ptr_array[j]) EQ 1) then begin
				;file lun - write result to file
				writeu, *out_ptr_array[j], continuum_struct.(j)
			endif else begin
				;in memory cube - store result
				(*out_ptr_array[j])[*,i,*] = continuum_struct.(j)
			endelse
		endif
	endfor

endfor

envi_tile_done, in_tile_id

envi_report_init, base = status_base, /finish



;============================
;    HEADER PREP
;============================

if ((n_elements(def_bands) eq 3) && (min(def_bands) gt 0)) then begin
	; For header... see note in write_cat_header prolog about 
	; decrementing def_bands
	def_bands -= 1
endif else begin
	def_bands = [0,0,0]
endelse

; Update input file list:
if (n_elements(cat_input_files) gt 0) then cat_input_files = strtrim(cat_input_files,2)
if (  (n_elements(cat_input_files) gt 0) && (cat_input_files[0] ne '-1') $
   && (cat_input_files[0] ne '')) then begin
	cat_input_files = [cat_input_files, file_basename(in_filename)]
endif else begin
	cat_input_files = [file_basename(in_filename)]
endelse


;Register memory item in ENVI session or close output file and write ENVI header
for j = 0, n_elements(result_select)-1 do begin
    ;if parameter cube is requested, set default band(s) to load to one, band names to parameter names
    if (j EQ 2) then begin
    	def_bands = [1]
    	bnames = ['P0', 'P1']
    endif

	if (result_select[j] EQ 1) then begin
		if (n_elements(*out_ptr_array[j]) EQ 1) then begin
			;file lun - close file and write header
			free_lun, *out_ptr_array[j]

			; Append continuum ops to history:
			contin_ops = ['CON','RAT','CPR']
			this_cat_history = cat_append_history(cat_history, contin_ops[j])

			stat = write_cat_header(FNAME=continuum_info_struct.outfm_result_array[j].name, $
			               NS=ns, NL=nl, NB=result_nb_array[j], $
			               DATA_TYPE=4, INTERLEAVE=1, OFFSET=0, $
			               BBL=bbl, BNAMES=bnames, DATA_IGNORE_VALUE=data_ignore_value, $
			               DEF_BANDS=def_bands, DESCRIP='CAT CRISM DATA', FWHM=fwhm, $
			               MAP_INFO=map_info_struct, $
			               WAVELENGTH_UNIT=wu, WL=wl, $
			               CAT_START_TIME          = cat_start_time, $
			               CAT_SCLK_START          = cat_sclk_start, $
			               CAT_CRISM_OBSID         = cat_crism_obsid, $
			               CAT_OBS_TYPE            = cat_obs_type, $
			               CAT_PRODUCT_VERSION     = cat_product_version, $
			               CAT_CRISM_DETECTOR_ID   = cat_crism_detector_id,  $
			               CAT_BIN_MODE            = cat_bin_mode, $
			               CAT_WAVELENGTH_FILTER   = cat_wavelength_filter, $
			               CAT_CRISM_DETECTOR_TEMP = cat_crism_detector_temp, $
			               CAT_CRISM_BENCH_TEMP    = cat_crism_bench_temp, $
			               CAT_CRISM_HOUSING_TEMP  = cat_crism_housing_temp, $
			               CAT_SOLAR_LONGITUDE     = cat_solar_longitude, $
			               CAT_PDS_LABEL_FILE      = cat_pds_label_file, $
			               CAT_SPECTRUM_RESAMPLED  = cat_spectrum_resampled, $
			               CAT_TILE_WAVE_FILE      = cat_tile_wave_file, $
			               CAT_SWEETSPOT_WAVE_FILE = cat_sweetspot_wave_file, $
			               CAT_WA_WAVE_FILE        = cat_wa_wave_file,  $
			               CAT_IR_WAVES_REVERSED   = cat_ir_waves_reversed, $
			               CAT_HISTORY             = this_cat_history, $
			               CAT_INPUT_FILES         = cat_input_files)

			; Report problems with header if any indicated by status returned:
			if (stat ne 0) then begin
				if (stat eq -1) then begin
					msg = 'Problem writing header for '+file_basename(atp_setup.datafile) + string(10b) $
					     + 'Required keywords for write_cat_header undefined.'
					ok = dialog_message(msg, title='Header Error')
				endif else if (stat eq -2) then begin
					msg = 'Problem writing header for '+file_basename(atp_setup.datafile) + string(10b) $
					     + 'Header already exists.'
					ok = dialog_message(msg, title='Header Error')
				endif else begin
					msg = 'Problem writing header for '+file_basename(atp_setup.datafile) + string(10b) $
					     + 'Unspecified error.'
					ok = dialog_message(msg, title='Header Error')
				endelse
			endif

		endif else begin
			;in memory cube - register with ENVI 
			envi_enter_data, (*out_ptr_array[j]), data_ignore_value = data_ignore_value, $
			               wl = wl, wavelength_unit=wu, bnames = bnames, $
			               bbl = bbl, fwhm=fwhm, def_bands=def_bands, $
			               descrip='CAT CRISM DATA', map_info = map_info_struct
		endelse

	endif
	;destroy pointers
	ptr_free, out_ptr_array[j]
endfor

END
