function mro_crism_lookupwv, wv, wvt, w=w


;-----------------------------------------------------------------------
; ABSTRACT
; function to find correct row for target wavelength
;
; INPUTS
; wv = target wavelength (nanometers)
; det = detector (0 = VNIR  1 = IR)
; lut = look up table array (3 by x) (Detector row, VNIR wvln, IR wvln)
; w = toggle return of wavelength rather than row.
;
; OUTPUT
; Returns row number closest to target wavelength
;
; Note that wavelengths outside the min/max of the detector range will
; return row numbers for the closest wavelength in the detector.
;
; HISTORY
; created 20006/04/07 by NRI for CRISM Summary Product support
;
; Oct 05,2007 MFM 
;	--Replace row search with more efficient min(abs(),row) construct.
;
;-----------------------------------------------------------------------


; Subtract wavelength from the values in the detector.
; Take absolute value, find minimum.
; That's the closest row to the target wavelength in the detector.


dum = min(abs(wvt - wv), row)
;row = where( abs(wvt - wv) EQ min(abs(wvt - wv)) )

if (keyword_set(w)) then out = wvt[row] else out = row

return, out
end