; mro_crism_summary_flatten.pro; 05/30/2007	(fps)
;   Code inherited from S. Murchie. Meadian-based column-normalization for spectral summary parameters (MSP mapping)
;   Renamed summary_flatten.pro to mro_crism_summary_flatten.pro
;   Converted procedure to function - summary_flat is now returned
;   Detection of input cube dimensions internal to procedure - no need to pass in as parameters
;   Removed variable = 0 statements; local memory freed on function return.
;   Replaced most for loops with IDL native function calls, e.g. median(X, dimension=Y)
;   Vectorized inner correction loop - speed enhancement
;   Note: scene is a BSQ array nsamples,nlines,nparams
;
; Modification History:
;

function mro_crism_summary_flatten, scene

scene_size = size(scene, /struct)

scene_columns = scene_size.dimensions[0]
scene_lines = scene_size.dimensions[1]
scene_bands = (scene_size.dimensions[2] > 1)

; Calculate a median summary parameter image for the scene
; where x is cross-track pixel and z is the summary parameter band.

medianframe = median(scene, dimension = 2)

; For each of z summary parameters calculate its median value.

medianband = median(medianframe, dimension = 1)

; Now add to each summary x,y position in each of z summary parameters
; the difference between the median of the column and the median of all columns.

summary_flat=fltarr(scene_columns,scene_lines,scene_bands)

for x=0,scene_columns-1 do begin
    for y=0,scene_lines-1 do begin
    	summary_flat[x,y,*] = scene[x,y,*]-(medianframe[x,*]-medianband)
    endfor
endfor

; Assign 65535.0 to all locations in the output scene that were 65535.0 in the input scene

indx = where(scene EQ 65535.0, num_indx)
if (num_indx GT 0) then begin
    summary_flat[indx] = 65535.0
endif


return, summary_flat

end


