;******************************************************************************
;
; pro cat_envi_open_ifnot, imgfile, r_fid, already_open, $
;                   no_realize=no_realize, no_interactive_query=no_interactive_query
;
; If a file is not already open, open it, and return flag whether it was 
; open before or not.
;
; INPUT:
;	imgfile		   File to open, with path
;
; OUTPUT PARAMETERS:
;	r_fid		   ENVI file ID
;	already_open   int = 1 if file was already open, 0 if not.
;
; KEYWORDS:
;	no_realize               Passes through to envi_open_file             
;	no_interactive_query     Passes through to envi_open_file 
;
; HISTORY:
;	2009 Nov 12   F. Morgan: First version.
;	2009 Dec 02   F. Morgan: Add missing argument trap.
;
;******************************************************************************

pro cat_envi_open_ifnot, imgfile, r_fid, already_open, invisible=invisible, $
                      no_realize=no_realize, no_interactive_query=no_interactive_query

; Make sure input path fully qualified
r_fid = -1
already_open = 0
if (n_elements(imgfile) eq 0) then return
fnms = file_search(imgfile, /fold_case, /fully_qualify_path)
if (fnms[0] eq '') then return   ; Can't even find the input
fnm_up = strupcase(strtrim(fnms[0],2))

; Is the file open already?
already_open = 0
current_ids = envi_get_file_ids()
if (current_ids[0] ne -1) then begin
	for i=0,n_elements(current_ids)-1 do begin
		envi_file_query, current_ids[i], fname=this_file
		if (size(this_file,/type) eq 7) then begin
			fnms = file_search(this_file, /fold_case, /fully_qualify_path)
			if (strupcase(strtrim(fnms[0],2)) eq fnm_up) then begin
				; This is the file...
				r_fid = current_ids[i]
				already_open = 1
				break
			endif
		endif
	endfor
endif

if (~already_open) then begin
	; open it...
	envi_open_file, imgfile, r_fid=r_fid, no_realize=no_realize, invisible=invisible, $
	         no_interactive_query=no_interactive_query
endif

return
end
