;***************************************************************************************
;
; function cat_is_digit, input_string
;
; Determine which characters in a string are numeric digits.
;
; INPUT PARAMETERS:
;	input_string	string		Scalar string
;
; OUTPUT PARAMETERS: None
;
; RETURN VALUE:
;	byte array with length = input string length,
;	elements that match digits = 1,
;	non-digit characters = 0
;
; KEYWORDS
;	allow_hex    If set, alos accept a..f, A..F as digits
;
; EXAMPLES:
;	x = cat_is_digit('g')
;		result: x = [0]
;
;	x = cat_is_digit('5')
;		result: x = [1]
;
;	x = cat_is_digit(';')
;		result: x = [0]
;
;	x = cat_is_digit("I haven't seen 50 cats!")
;		result: x = [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0]
;
;	x = cat_is_digit("I haven't seen 50 cats!",/allow_hex) & help,x & Print,x
;		result: x = [0,0,0,1,0,1,0,0,0,0,0,1,1,0,0,1,1,0,1,1,0,0,0]
;
; HISTORY:
;	2009 Feb 09   FM: Change name to cat_is_digit to avoid conflict with similarly 
;	              named procedures
;	2009 Apr 22   FM: Documentation edit 
;	2009 Jun 12   FM: Add hex character capability
;	2009 Nov 04   FM. Add allow minus keyword.
;	2009 Nov 04   FM. Add allow_float.
;
;***************************************************************************************

function cat_is_digit, input_string, allow_hex=allow_hex, allow_minus=allow_minus, $
                 allow_float=allow_float

n = strlen(input_string)
if (n eq 0) then return,0
x = bytarr(n)

; Valid digits
if (keyword_set(allow_hex)) then begin
	valid_digits = [byte('0123456789'), byte('ABCDEF'), byte('abcdef')]
endif else begin
	valid_digits = [byte('0123456789')]
endelse
if (keyword_set(allow_float)) then begin
	valid_digits = [valid_digits, byte('-'), byte('.')]
endif else if (keyword_set(allow_minus)) then begin
	valid_digits = [valid_digits, byte('-')]
endif

; use ascii values to identify alphabetic characters: either A-Z or a-z:
b = byte(input_string)
n = n_elements(b)
for i=0,n-1 do begin
	k = where((valid_digits eq b[i]),nk)
	if (nk ne 0) then x[i]=1B else x[i]=0B
endfor

return,x
end
