pro cat_scale_zprofile, plot_obj, oAxes, wave, spec

; Apply scaling to a z-profile plot.
;
; INPUT:
;	plot_obj   object      Plot object
;	oAxes      struct      Axes structure {xmin, xmax, ymin, ymax, yscl}
;	wave       fltarr(nb)  Wavelength array
;	spec       fltarr(nb)  Spectrum (y values)
;
; SIDE EFFECT:
;	Resets plot axes if needed


YPCT = [0.03, 0.95]   ; y auto-scale percentiles

if (((plot_obj.xrange)[0] ne oAxes.xmin) or ((plot_obj.xrange)[1] ne oAxes.xmax)) then begin
	plot_obj.xrange = [oAxes.xmin, oAxes.xmax]
endif

; y-scaling depends on auto vs. absolute
if (oAxes.yscl eq 1) then begin
	; Absolute scaling; just set directly
	if (((plot_obj.yrange)[0] ne oAxes.ymin) or ((plot_obj.yrange)[1] ne oAxes.ymax)) then begin
		plot_obj.yrange = [oAxes.ymin, oAxes.ymax]
	endif
endif else begin
	; Auto scale; set yrange according to data
	k = where(((wave ge oAxes.xmin) and (wave le oAxes.xmax) and (spec lt 65.0)), nk)
	if (nk gt 4) then begin
		active_spec = spec[k]
		active_spec = active_spec[sort(active_spec)]
		;idx5 = (round(min(YPCT)*nk)) > 1      ; drop at least 1 small value
		idx95 = (round(max(YPCT)*nk)) < (nk-2)   ; drop at least 1 large value
		ylo = floor(99.0*oAxes.ymin*active_spec[idx95]) / 100.0
		yhi = ceil(101.0*oAxes.ymax*active_spec[idx95]) / 100.0
		plot_obj.yrange = [ylo, yhi]
	endif
endelse

return
end
