;******************************************************************************
;
; function extract_dr_sclk, dr_fnm
;
; Extract validity SCLK from data record filenames (CDR or ADR).
;
; INPUT:
;	dr_fnm      str[arr]         Filenames for CDR's or ADR's
;
; OUTPUT PARAMETERS: None
;
; RETURN:
;	Array of SCLK structures:
;		sclk[].part = integer partition   (-1 if fails to read partition)
;		sclk[].tick = long tick  (-1 if fails to read tick)
;
; KEYWORDS: None.
;
; NOTES:
;
; USES:
;	cat_all_digits
;
; HISTORY:
;	2009 Sep 03 FM  First version
;
;******************************************************************************

function extract_dr_sclk, dr_fnm

forward_function cat_all_digits

nf = n_elements(dr_fnm)
sclk = replicate( {part: -1, tick: -1L}, nf)
for i=0,nf-1 do begin
	; Upper case, basename, drop extension:
	fnm = strupcase(strtrim(file_basename(dr_fnm[i]),2))
	p = strpos(fnm,'.', /reverse_search)
	if (p gt 0) then fnm=strmid(fnm,0,p)

	; ADR or CDR?
	xcode = strmid(fnm,0,3)

	spart = ''
	stick = ''
	if (xcode eq 'ADR') then begin
		if (strmid(fnm, 21, 2) eq 'VS') then begin
			spart = strmid(fnm,3,1)
			stick = strmid(fnm,4,10)
		endif else begin
			; Other ADRs handled according to PDS spec:
			spart = strmid(fnm,12,1)
			stick = strmid(fnm,14,10)
		endelse
	endif else if (xcode eq 'CDR') then begin
		xlvl = strmid(fnm, 3,1)
		if (xlvl eq '4') then begin
			; CDR4 filename is CDR4pcccccccccc_...
			spart = strmid(fnm,4,1)
			stick = strmid(fnm,5,10)
		endif else if (xlvl eq '6') then begin
			; CDR6 filename is CDR6_p_cccccccccc_...
			spart = strmid(fnm,5,1)
			stick = strmid(fnm,7,10)
		endif
	endif

	if ((strlen(spart) ne 1) || (strlen(stick) ne 10) ) then continue        ; leaves values -1
	if ((~cat_all_digits(spart)) || (~cat_all_digits(stick))) then continue  ; leaves values -1
	sclk[i].part = fix(spart)
	sclk[i].tick = long(stick)
endfor

return, sclk
end
