;******************************************************************************
;
; function cat_file_check, filepath, found_files_list
;
; Test for existence of a file, allowing case difference. 
;
; INPUT:
;	filepath		File path; may be basename only or include full path.
;
; OUTPUT PARAMETERS:
;	found_file_list		strarr		All filepaths found. [''] if none.
;
; RETURN VALUE:  INT = number of files found; 0 -> no files found.
;
; KEYWORDS: none
;
; NOTES:
;	Assumptions:
;
; USES:
;
; HISTORY:
;	2008 Apr 04   F. Morgan: First version written.
;	2009 Feb 02   F. Morgan: Trap undefined filepath input.
;	2009 Jul 07   F. Morgan: Overhaul - use file_search() instead of 
;	              all-case testing; return = #files instead of code.
;	              (Was never using that anyway)
;	2009 Nov 12   F. Morgan: Make sure filepath=string type to prevent crash.
;
;******************************************************************************

function cat_file_check, filepath, found_files_list

if (n_elements(filepath) eq 0) then return,0

if ((strtrim(filepath) eq '') || (size(filepath,/type) ne 7)) then begin
	; Have to trap this case because file_dirname('') returns '.' 
	; and you'd end up with a list of default directories + empty string
	; Also, if not string type, crashes file_search.
	found_files_list = ['']
	return,0
endif

found_files_list = file_search(filepath, /fold_case)
if (found_files_list[0] eq '') then begin
	found_files_list = ['']
	nfiles = 0
endif else begin
	nfiles = n_elements(found_files_list)
endelse

return, nfiles
end
