;******************************************************************************
;
; function pdsfile_is_mtrdr, allow_unproj=allow_unproj
;
; Given the filename of a PDS file, determine whether it is a CRISM 
; MTRDR or not.
;
; INPUT:
;	fnm		string		File path+name (path not used if present)
;
; OUTPUT:
;	Parameters: none
;	Return value: 0, not tile; 1, is a tile
;
; KEYWORDS:
;	allow_unproj    If set, also returns 1 for unproject targeted RDR file.
;
; NOTES:
;	Returns 1 (true) iff the input file is a MTRDR. 
;
;	THE FILE IS ASSUMED TO EXIST AND TO BE A PDS FILE, or at least to have a name
;	descended from PDS convention (in which a tile file uniquely includes 'MRR').
;
; USES:
;	No external procedures.
;
; HISTORY:
;	2012 Mar 15 F. Morgan: created
;
;******************************************************************************

function pdsfile_is_mtrdr, fnm, allow_unproj=allow_unproj

if (strpos(strupcase(file_basename(fnm)),'_MTR') ge 0) then return,1
if (keyword_set(allow_unproj)) then begin
	if (strpos(strupcase(file_basename(fnm)),'_TER') ge 0) then return,1
endif
return,0
end
