;******************************************************************************
;
; function get_tile_sensid
;
; Get sensor ID for bands from a CRISM tile.
;
; INPUT:
;	fnm		string		Filename of file from which tile was read.
;	waves	fltarr		Band wavelengths
;
; OUTPUT:
;	Parameters:    None
;	Return value:  Vector with 1-to-1 correspondence to the input wavelength vector, 
;                  with 1's (good bands) and 0's (bad bands).
;	               Returns -1 on failure.
;
; KEYWORDS: none
;
; NOTES:
;	Input 'wu' not used.
;
; USES:
;	load_tile_waves
;	crism_waves2nm
;
; HISTORY:
;	2007 Nov 26   F. Morgan: First version written and tested.
;	2009 Jan 01   F. Morgan: add pds_row argument to load_tile_waves
;	2009 Mar 31   F. Morgan: Don't trust wavelength units; and use crism_waves2nm
;	              for nm conversion, with initial units check in routine
;	2009 Jul 27   F. Morgan: Documentation update
;	2009 Nov 02   F. Morgan: Eliminate unused wavelength unit argument.
;
;******************************************************************************

function get_tile_sensid, fnm, waves

forward_function load_tile_waves, crism_waves2nm

; Load original PDS tile wavelengths and sensor ID:
stat = load_tile_waves(fnm, pds_wv, pds_row, pds_sid)
if (stat ne 0) then return, -1

; If inputs are microns, convert to nm
wvnm = crism_waves2nm(waves)

; Current dataset of interest may have been reduced from original PDS
; wavelength set. Loop through all input wavelengths, assign sensor ID 
; from the PDS table entry with the closest wavelength.
nw = n_elements(wvnm)
sid = intarr(nw)
for i=0,nw-1 do begin
	dum = min(abs(pds_wv - wvnm[i]), j)
	sid[i] = pds_sid[j]
endfor

return, sid
end
