;******************************************************************************
;
; function mid_valid_value
;
; CAT/ENVI function to get the midpoint of valid values in an array
;
; INPUT:
;	data		Data array
;	max_limit	Values beyond this limit are not considered
;
; OUTPUT PARAMETERS:
;	none
;
; RETURN VALUE:  Midpoint value, or badflag
;	
; KEYWORDS:
;	min_limit		Minimum value to consider valid
;	max_limit		Maximum value to consider valid
;	badflag			Value to return to flag case when all pixels exceed limits; default=0
;
; HISTORY:
;	2008 Apr 02   F. Morgan: First version written.
;
;******************************************************************************

function mid_valid_value, data, min_limit=min_limit, max_limit=max_limit, badflag=badflag


fpmax = (machar(/double)).xmax

if (n_elements(min_limit) eq 0) then min_limit = -fpmax
if (n_elements(min_limit) eq 0) then max_limit = fpmax
if (n_elements(badflag) eq 0) then badflag = 0


k = where(((data ge min_limit) and (data le max_limit)), nk)
if (nk eq 0) then return, badflag

return, 0.5 * (min(data[k]) + max(data[k]))
end
