;******************************************************************************
;
; function pick_cdr_for_obs, obsid, cdr_list, type_idx
;
; Select a CDR for a observation from ID-indexed list.
;
; INPUT:
;	obsid      float or long       Observation ID 
;	cdr_list   struct-arr          List of CDR's; 
;                                      cdr_list.id = obs ID
;	                                   cdr_list.fnm = CDR filenames for  
;	type_idx   int                 Index into filename array in the CDR list for the desired type
;
; OUTPUT PARAMETERS: None
;
; RETURN VALUE:   Chosen CDR flename; '' if fail
;
; KEYWORDS: none
;
; NOTES:
;	Expected usage:
;		Selection of CDR from list generated by autolist_tile_cdr4s; 
;		Example to pick a AT CDR for observation [i,j] in array obsid...
;			stat = autolist_tile_cdr4s(obsid, ['WA','AT'], 'L', 3, 1, cdr_list, nfail)
;			cdr_for_ij = pick_cdr_for_obs(obsid[i,j], cdr_list, 1)
;		... using 1 for type_idx since 'AT' was at index 1 in the cdr_type input to 
;		autolist_tile_cdr4s
;
;	Input OBSID should be scalar; if not, uses first element and ignores rest.
;
; USES:
;
; HISTORY:
;	2009 Sep 19  F. Morgan: First version.
;
;******************************************************************************

function pick_cdr_for_obs, obsid, cdr_list, type_idx


k = where((cdr_list.id eq obsid[0]), nk) 
if (nk eq 0) then return,''                                ; no matching observations in list
cdrs_for_obs = cdr_list[k[0]].fnm
if (type_idx ge n_elements(cdrs_for_obs)) then return,''   ; not enough filenames in list
return,cdrs_for_obs[type_idx]
end
